﻿using System;
using System.ComponentModel;
using System.Data;
using System.Reflection;
using System.Windows.Forms;
using Telerik.WinControls.UI;

namespace SummaryGrid
{
    public partial class SummaryGridForm : Form
    {
        private readonly static MethodInfo BuildSummaryRowsMethod;

        public SummaryGridForm()
        {
            InitializeComponent();
        }

        static SummaryGridForm()
        {
            BuildSummaryRowsMethod = typeof(GridViewGroupRowInfo).GetMethod("BuildSummaryRows", BindingFlags.NonPublic | BindingFlags.Instance);
        }

        protected override void OnLoad(System.EventArgs e)
        {
            base.OnLoad(e);

            this.summaryGrid.LoadFrom(this.GetSchema().CreateDataReader());
            this.summaryGrid.ReadOnly = true;
            this.summaryGrid.EnableGrouping = false;

            DataTable table = this.GetSchema();
            table.Rows.Add(DateTime.Now.Date, 500);
            table.Rows.Add(DateTime.Now.Date, 1000);
            table.Rows.Add(DateTime.Now.AddDays(-50).Date, 200);

            this.parentGrid.DataSource = table;
            this.parentGrid.GroupDescriptors.Add("Date", ListSortDirection.Ascending);

            this.parentGrid.SummaryRowsBottom.Add(
                new GridViewSummaryRowItem(
                    new GridViewSummaryItem[] 
                    {
                        new GridViewSummaryItem("Balance", "Total: {0}", GridAggregateFunction.Sum)
                    })
                );

            this.RebuildSummaryGrid();
            this.parentGrid.MasterTemplate.ViewChanged += this.OnViewChanged;
        }

        private DataTable GetSchema()
        {
            DataTable table = new DataTable("table");
            table.Columns.Add("Date", typeof(DateTime));
            table.Columns.Add("Balance", typeof(int));
            return table;
        }

        private void OnViewChanged(object sender, DataViewChangedEventArgs args)
        {
            ViewChangedAction action = args.Action;

            if (action == ViewChangedAction.Reset || action == ViewChangedAction.GroupingChanged ||
                action == ViewChangedAction.Add || action == ViewChangedAction.Remove || action == ViewChangedAction.ItemChanged ||
                action == ViewChangedAction.FilteringChanged || action == ViewChangedAction.SortingChanged)
            {
                this.RebuildSummaryGrid();
            }
        }

        private void RebuildSummaryGrid()
        {
            this.summaryGrid.Rows.Clear();

            foreach (GridViewRowInfo row in this.parentGrid.ChildRows)
            {
                GridViewGroupRowInfo group = row as GridViewGroupRowInfo;

                if (group == null)
                {
                    return;
                }

                BuildSummaryRowsMethod.Invoke(group, null);

                object date = (group.Group.Key as object[])[0];
                object balance = group.BottomSummaryRows[0].SummaryRowItem[0].Evaluate(group);

                this.summaryGrid.Rows.Add(date, balance);
            }
        }
    }
}
